package org.freehep.jas.plugin.pluginmanager;

import java.awt.Component;
import java.io.IOException;
import org.freehep.application.PropertyUtilities;
import org.freehep.application.studio.Studio;
import org.freehep.jas.services.PreferencesTopic;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.xml.menus.XMLMenuBuilder;
import java.net.URL;
import java.util.Collection;
import java.util.Properties;
import javax.swing.JComponent;
import org.freehep.application.studio.PluginInfo;
import org.xml.sax.SAXException;

/**
 * A Plugin which provides a Plugin Manager.
 *
 * @author tonyj
 * @version $Id: PluginManager.java 14736 2013-05-01 20:11:12Z onoprien $
 */
public class PluginManager extends org.freehep.application.studio.pluginmanager.PluginManager implements PreferencesTopic {

    private static String[] preferencesPath = {"Plugin Manager"};

    @Override
    protected void init() throws SAXException, IOException {
        super.init();
      
        final Studio app = getApplication();
        Properties user = app.getUserProperties();
        String pluginURL = user.getProperty("PluginManager.URL");
        setCheckURL(pluginURL == null ? null : new URL(pluginURL));
        setCheckAtStart(PropertyUtilities.getBoolean(user, "checkForPluginsAtStart", false));

        XMLMenuBuilder builder = app.getXMLMenuBuilder();
        java.net.URL xml = PluginManager.class.getResource("PluginManager.menus");
        builder.build(xml);

        app.getLookup().add(this);
        app.getCommandTargetManager().add(new Commands());
    }

    @Override
    /**
     * Override to always return true since some Jas3 plugins require init() to
     * have been run on all other plugins before their own postInit() runs.
     */
    public boolean install(Component parent, Collection<PluginInfo> plugins) {
      super.install(parent, plugins);
      return true;
    }

    @Override
    public boolean apply(JComponent panel) {
        return ((PluginPreferences) panel).apply();
    }

    @Override
    public JComponent component() {
        return new PluginPreferences(this);
    }

    @Override
    public String[] path() {
        return preferencesPath;
    }
    
// -- Command processor class : ------------------------------------------------
    
    class Commands extends CommandProcessor {
        public void onPluginManager() {
            showPluginManager();
        }
    }
    
}